@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015-2020 Cinc
+# Copyright (C) 2015-2021 Cinc
 #
 # License: 3-clause BSD
 #
@@ -14,6 +14,7 @@ from simplemultiproject.session import get_project_filter_settings, \
 from simplemultiproject.smp_model import SmpMilestone, SmpVersion
 from trac.config import OrderedExtensionsOption
 from trac.core import *
+from trac.util.translation import _
 from trac.web.api import IRequestFilter
 from trac.web.chrome import add_script, add_script_data, add_stylesheet, Chrome, ITemplateProvider
 
@@ -53,7 +54,26 @@ class SmpRoadmapModule(Component):
         if self.pre_1_3:
             self.group_tmpl = chrome.load_template("smp_roadmap.html", None)
         else:
-            self.group_tmpl = chrome.load_template("roadmap_jinja.html", False)
+            self.group_tmpl = chrome.load_template("smp_roadmap_jinja.html", False)
+
+    # Unused preference item
+    # if get_filter_settings(req, 'roadmap', 'smp_hideprojdesc'):
+    #     hide.append('projectdescription')
+    # prefs = """<div>
+    #               <input type="checkbox" id="hideprojectdescription" name="smp_hideprojdesc" value="1"
+    #                      {hideprjdescchk} />
+    #               <label for="hideprojectdescription">{hideprjdesclabel}</label>
+    #         </div>"""
+
+    def create_hide_milestone_item(self, req):
+        prefs = """
+        <div>
+              <input type="checkbox" id="hidemilestones" name="smp_hidemilestones" value="1"
+                     {hidemschk} />
+              <label for="hidemilestones">{hidemslabel}</label>
+        </div>"""
+        hidemschk = ' checked="checked"' if get_filter_settings(req, 'roadmap', 'smp_hidemilestones') else '',
+        return prefs.format(hidemslabel=_('Hide milestones'), hidemschk=hidemschk)
 
     # IRequestFilter methods
 
@@ -91,11 +111,9 @@ class SmpRoadmapModule(Component):
                                                  u'value="1"%s/>'
                                                  u'<label for="groupbyproject">Group by project</label></div><br />' %
                                                  chked))
-
-                # Change label to include versions
-                # xpath: //label[@for="showcompleted"]
-                # xform = JTransformer('label[for=showcompleted]')
-                # filter_list.append(xform.replace(u'<label for="showcompleted">Show completed milestones</label>'))
+                # Add preference checkbox
+                xform = JTransformer('#prefs div.buttons')
+                filter_list.append(xform.before(self.create_hide_milestone_item(req)))
 
                 if chked:
                     if not self.group_tmpl:
@@ -114,9 +132,6 @@ class SmpRoadmapModule(Component):
                 add_script_data(req, {'smp_filter': filter_list})
                 add_script(req, 'simplemultiproject/js/jtransform.js')
 
-                data.update({'show': req.args.get('show', [])  # TODO: is this used at all?
-                             })
-
         return template, data, content_type
 
     # IRoadmapDataProvider
@@ -124,57 +139,46 @@ class SmpRoadmapModule(Component):
     def add_projects_to_dict(self, req, data):
         """Add allowed projects to the data dict.
 
-        This checks if the user has access to a project. If not the project won't be added.
-        """
-        # Get all projects user has access to.
-        usr_projects = []
-        for project in Project.select(self.env):  # This is already sorted by name
-            if project.restricted:
-                if (PERM_TEMPLATE % project.id) in req.perm:
-                    usr_projects.append(project)
-            else:
-                usr_projects.append(project)
-        all_known_proj_ids = [project.id for project in usr_projects]
+        :param req: a Request object
+        :param data: dictionary holding data for template
+        :return None
 
+        This checks if the user has access to the projects. If not a project won't be added to the list of available
+        projects. Closed projects are ignored, too.
+        """
+        usr_projects = SmpPermissionPolicy.active_projects_by_permission(req, Project.select(self.env))
         data.update({'projects': usr_projects,
-                     'project_ids': all_known_proj_ids})
+                     'project_ids': [project.id for project in usr_projects]})
 
     def add_project_info_to_milestones(self, data):
         # Do the milestone updates
+        data['ms_without_prj'] = False
         if data.get('milestones'):
-            all_known_proj_ids = data['project_ids']
             # Add info about linked projects
             for item in data.get('milestones'):
+                # Used in smp_roadmap.html to check if there is a ms - proj link
                 ids_for_ms = self.smp_milestone.get_project_ids_for_resource_item('milestone', item.name)
                 if not ids_for_ms:
-                    # Used in smp_roadmap.html to check if there is a ms - proj link
-                    item.id_project = all_known_proj_ids  # Milestones without a project are for all
+                    item.id_project = []  # Milestones without a project are for all
+                    data['ms_without_prj'] = True
                 else:
                     item.id_project = ids_for_ms
 
-    def add_project_info_to_versions(self, data):
-        if data.get('versions'):
-            all_known_proj_ids = data['project_ids']
-            for item in data.get('versions'):
-                ids_for_ver = self.smp_version.get_project_ids_for_resource_item('version', item.name)
-                if not ids_for_ver:
-                    # Used in smp_roadmap.html to check if there is a version - proj link
-                    item.id_project = all_known_proj_ids  # Versions without a project are for all
-                else:
-                    item.id_project = ids_for_ver
-
     def add_data(self, req, data):
         # Get all projects user has access to.
         self.add_projects_to_dict(req, data)
         self.add_project_info_to_milestones(data)
-        self.add_project_info_to_versions(data)
+        # self.add_project_info_to_versions(data)
 
         return data
 
     def filter_data(self, req, data):
 
-        filter_proj = get_project_filter_settings(req, 'roadmap', 'smp_projects', 'All')
+        if data and get_filter_settings(req, 'roadmap', 'smp_hidemilestones'):
+            data['milestones'] = []
+            data['milestone_stats'] = []
 
+        filter_proj = get_project_filter_settings(req, 'roadmap', 'smp_projects', 'All')
         if 'All' in filter_proj:
             return data
 
@@ -202,10 +206,11 @@ class SmpRoadmapModule(Component):
                         if name in filter_proj:
                             filtered_items.append(ms)
                             filtered_item_stats.append(item_stats[idx])
-                            break  # Only add a milstone once
+                            break  # Only add a milestone once
             data['milestones'] = filtered_items
             data['milestone_stats'] = filtered_item_stats
 
+        # TODO: Is this still needed?
         if 'versions' in data:
             item_stats = data.get('version_stats')
             filtered_items = []
