@@ -66,7 +66,7 @@ class SmpBaseModel(object):
         """Update data sets for item 'name' and the list of projects
         id_projects.
 
-        @param name: the name of them item
+        @param name: the name of the item
         @param id_projects: single id of a project or list if ids
 
         Update is done by deleting the item from the SMP database and
@@ -84,12 +84,6 @@ class SmpBaseModel(object):
                 SELECT %s, id_project FROM smp_%s_project
                 """ % (resource_name, resource_name))
 
-    def _update(self, resource_name, name, id_projects):
-
-        self.env.db_transaction("""
-            UPDATE smp_%s_project SET id_project=%%s WHERE %s=%%s
-            """ % (resource_name, resource_name), (id_projects, name))
-
     def get_project_names_for_item(self, name):
         """Get a list of project names the item
         (version, milestone, component) is associated with.
@@ -208,9 +202,9 @@ class SmpProject(SmpBaseModel):
         prj_id = None
         with self.env.db_transaction as db:
             cursor = db.cursor()
-            cursor.execute("""INSERT INTO smp_project (name, summary, description, closed, restrict_to) 
+            cursor.execute("""INSERT INTO smp_project (name, summary, description, closed, restrict_to)
                            VALUES(%s,%s,%s,%s,%s)""", (name , summary, description, closed, restrict_to))
-            prj_id = db.get_last_id(cursor, 'smp_project')
+            prj_id = db.get_last_id(cursor, 'smp_project', 'id_project')
 
         # keep internal ticket custom field data up to date
         self.get_all_projects()
@@ -225,12 +219,6 @@ class SmpProject(SmpBaseModel):
         if type(prj_id) is not list:
             prj_id = [prj_id]
 
-        with self.env.db_transaction as db:
-            for prj in prj_id:
-                for res in ('component', 'milestone', 'version'):
-                    db("""DELETE FROM smp_%s_project WHERE id_project=%%s""" % res, (prj,))
-                db("""DELETE FROM smp_project WHERE id_project=%s""", (prj,))
-
         # remove permission from user
         permsys = PermissionSystem(self.env)
         for project_id in prj_id:
@@ -239,6 +227,12 @@ class SmpProject(SmpBaseModel):
             for user in users:
                 permsys.revoke_permission(user, permission)
 
+        with self.env.db_transaction as db:
+            for prj in prj_id:
+                for res in ('component', 'milestone', 'version'):
+                    db("""DELETE FROM smp_%s_project WHERE id_project=%%s""" % res, (prj,))
+                db("""DELETE FROM smp_project WHERE id_project=%s""", (prj,))
+
         # keep internal ticket custom field data up to date
         self.get_all_projects()
 
@@ -261,7 +255,7 @@ class SmpProject(SmpBaseModel):
                                          (prj_id, ))[0][0]
             except IndexError:
                 return
-            db("""UPDATE smp_project 
+            db("""UPDATE smp_project
                   SET name=%s, summary=%s, description=%s, closed=%s, restrict_to=%s
                   WHERE id_project=%s""", (name, summary, desc, closed, restrict_to, prj_id))
             if old_name != name:
@@ -304,17 +298,19 @@ class SmpProject(SmpBaseModel):
             return self.env.db_query("""SELECT id_project FROM smp_project WHERE name = '%s'""" %
                                      (project_name, )) != []
 
-    def apply_user_restrictions(self, projects, username):
+    def apply_user_restrictions(self, projects, username=None, perms=None):
         """Create a new list of projects the user has access to.
 
         :param projects: list of projects. each item is a tuple with all known project informations
         :param username: name of the current user
+        :param perms: PermissionCache object
         :return: a new filtered list of projects
 
         Filtering is done by checking if the user has permission PROJECT_<project_id>_MEMBER for any project
         in the project list.
         """
-        perms = PermissionSystem(self.env).get_user_permissions(username)
+        if not perms:
+            perms = PermissionSystem(self.env).get_user_permissions(username)
 
         # Note that the permission may be present but set to False
         return [prj for prj in projects if not prj.restricted or
@@ -325,7 +321,7 @@ class SmpProject(SmpBaseModel):
         projects = self.get_all_projects()
         prj_cache = {project.name: project for project in projects}
 
-        for project_name in self.env.db_query("""SELECT value FROM ticket_custom 
+        for project_name in self.env.db_query("""SELECT value FROM ticket_custom
                                             WHERE name='project' AND ticket = %s""" % (tkt_id,)):
             name = project_name[0]
             try:
@@ -373,10 +369,6 @@ class SmpVersion(SmpBaseModel):
         # TODO: this shouldn't be called with the resource name
         return self._get_all_names_and_project_id_for_resource('version')
 
-    def update_project_id_for_version(self, version_name, id_project):
-        # TODO: this shouldn't be called with the resource name
-        self._update('version', version_name, id_project)
-
     def get_versions_for_project_id(self, id_project):
         """Get versions for the given project id.
 
@@ -385,3 +377,11 @@ class SmpVersion(SmpBaseModel):
                  May be empty.
         """
         return self.get_items_for_project_id(id_project)
+
+    def get_project_ids_for_version(self, version_name):
+        """Get a list of project ids his version is associated with
+
+        :param version_name: name of a version
+        :return: list of project ids
+        """
+        return self.get_project_ids_for_resource_item(self.resource_name, version_name)
