@@ -6,14 +6,14 @@
 #
 import unittest
 
-from trac.test import EnvironmentStub
+from trac.test import EnvironmentStub, MockPerm
 from simplemultiproject.environmentSetup import smpEnvironmentSetupParticipant
-from simplemultiproject.smp_model import SmpProject
+from simplemultiproject.smp_model import SmpMilestone, SmpProject
 from simplemultiproject.permission import SmpPermissionPolicy
 from simplemultiproject.tests.util import revert_schema
 
 
-class TestPermisssionProvider(unittest.TestCase):
+class TestPermissionProvider(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True,
@@ -23,11 +23,12 @@ class TestPermisssionProvider(unittest.TestCase):
             smpEnvironmentSetupParticipant(self.env).upgrade_environment(db)
         self.prj_model = SmpProject(self.env)
         self.perm_plugin = SmpPermissionPolicy(self.env)
+        self.ms_model = SmpMilestone(self.env)
 
     def tearDown(self):
         self.env.reset_db()
 
-    def test_permission_no_projects(self):
+    def test_default_permissions_no_projects(self):
         """This should be [PROJECT_SETTINGS_VIEW, (PROJECTS_ADMIN, [PROJECT_SETTINGS_VIEW])]"""
         perms = self.perm_plugin.get_permission_actions()
         self.assertEqual(2, len(perms))
@@ -39,7 +40,7 @@ class TestPermisssionProvider(unittest.TestCase):
             else:
                 self.assertEqual('PROJECT_SETTINGS_VIEW', item)
 
-    def test_permission_projects(self):
+    def test_permission_for_projects(self):
         prjs = ((1, 'p 1'), (2, 'p 2'), (3, 'p 3'), (4, 'p 4'))
         for p_id, name in prjs:
             prj_id = self.prj_model.add(name)
@@ -72,7 +73,7 @@ class TestPermisssionProvider(unittest.TestCase):
         self.assertEqual(5, len(perm_lst))
         self.assertSequenceEqual(perm_lst, expected)  # This will check for duplicates
 
-    def test_permission_projects_delete_prj(self):
+    def test_permission_for_projects_delete_prj(self):
         # We first create some projects. Then one project is removed. The new list of project member perms
         # must be visible
         prjs = ((1, 'p 1'), (2, 'p 2'), (3, 'p 3'), (4, 'p 4'))
@@ -108,7 +109,7 @@ class TestPermisssionProvider(unittest.TestCase):
         self.assertEqual(4, len(perm_lst))
         self.assertSequenceEqual(perm_lst, expected)  # This will check for duplicates
 
-    def test_permission_projects_delete_add_prj(self):
+    def test_permission_for_projects_delete_add_prj(self):
         # We first create some projects. Then one project is removed and added again. The project must have a new
         # id .The new list of project member perms must be returnd by the plugin
         prjs = ((1, 'p 1'), (2, 'p 2'), (3, 'p 3'), (4, 'p 4'))
@@ -147,10 +148,55 @@ class TestPermisssionProvider(unittest.TestCase):
         self.assertEqual(5, len(perm_lst))
         self.assertSequenceEqual(perm_lst, expected)  # This will check for duplicates
 
+    def test_check_milestone_permissions(self):
+        """Check permission handling for milestones"""
+
+        # A milestone is available for a user if
+        #  a) it's not associated with any project
+        #  b) is associated with a project and the user is member of the project
+        #  c) is associated with several projects and at least one of them is not restricted
+        #  d) is associated with a project without restrictions
+
+        prjs = ((1, 'p 1', 'YES'), (2, 'p 2', None), (3, 'p 3', 'YES'), (4, 'p 4', None),
+                (5, 'p 5', 'YES'), (6, 'p 6', 'YES'), (7, 'p 7', None), (8, 'p 8', None))
+        for p_id, name, restrict in prjs:
+            prj_id = self.prj_model.add(name, restrict_to=restrict)
+            self.assertEqual(p_id, prj_id)  # prj_id is auto increment
+        class Perm(MockPerm):
+            perms = ['PROJECT_SETTINGS_VIEW', 'TICKET_VIEW', 'PROJECT_1_MEMBER']
+            def has_permission_mock(self, action, realm_or_resource=None, id=False,
+                       version=False):
+                return action in self.perms
+            __contains__ = has_permission_mock
+
+        # Associate milestones with project ids
+        self.ms_model.add("foo1", 1)
+        self.ms_model.add("bar", 2)
+        self.ms_model.add("baz", 3)
+        self.ms_model.add("foo2", 1)
+        perm = Perm()
+        # This milestone has no project thus user can access it (a)
+        self.assertTrue(self.perm_plugin.check_milestone_permission('no_prj', perm))
+        # Milestone for restricted project 1 but user has permission PROJECT_1_MEMBER (b)
+        self.assertTrue(self.perm_plugin.check_milestone_permission('foo1', perm))
+        # Milestone for project 2 but the project has no restrictions thus user may access it (d)
+        self.assertTrue(self.perm_plugin.check_milestone_permission('bar', perm))
+        # Milestone for project 3 but the project has restrictions thus user may not access it
+        self.assertFalse(self.perm_plugin.check_milestone_permission('baz', perm))
+
+        # Check if restrictions are properly checked
+        self.ms_model.add("barbaz", 5)
+        self.ms_model.add("barbaz", 6)
+        self.assertFalse(self.perm_plugin.check_milestone_permission('barbaz', perm))
+        # Milestone is in additon associated with a single unrestricted project (c)
+        self.ms_model.add("barbaz", 7)
+        self.assertTrue(self.perm_plugin.check_milestone_permission('barbaz', perm))
+
+
 
 def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TestPermisssionProvider))
+    suite.addTest(unittest.makeSuite(TestPermissionProvider))
 
     return suite
 
